@@ -4,7 +4,7 @@
 #'
 #' @description
 #' **DoSA()** is a function for conducting sequential analysis.
-#'
+#'#'
 #' @param data    DATAFRAME consists of relevant information.
 #' @param source  CHARACTER for labeling the included data sets.
 #' @param time    NUMERIC values of time sequence.
@@ -33,17 +33,19 @@
 #'                the current version also supports "REML" and "PM" methods for
 #'                calculating heterogeneity estimator.
 #' @param pooling CHARACTER for indicating which method has to be used for pooling
-#'                binary data. Current version consists of "IV" and "MH" for
-#'                binary data pooling.
+#'                binary data. Besides, current version also supports "MH" and
+#'                "Peto" for binary data pooling.
 #' @param alpha   NUMERIC value between 0 to 1 for indicating the assumed type I
 #'                error.
 #' @param beta    NUMERIC value between 0 to 1 for indicating the assumed type II
 #'                error.
 #' @param PES     NUMERIC value for indicating the presumed meaningful effect size.
 #' @param PV      NUMERIC value for indicating the presumed variance of the
-#'                meaningful effect size.
-#' @param adjust  CHARACTER for indicating how to adjust expected information size.
-#'                Current version consists of "none" and "D2" for the adjustment.
+#'                meaningful effect size. Current version allows a numeric value,
+#'                "post-hoc", and "PES" based on different considerations.
+#' @param adjust  CHARACTER for indicating how to adjust optimal information size.
+#'                Current version consists of "none", "D2", "I2", "CHL", "CHM", and
+#'                "CHH" for the adjustment.
 #' @param plot    LOGIC value for indicating whether to illustrate alpha-spending
 #'                monitoring plot.
 #' @param id      LOGIC value for indicating whether to label each data source.
@@ -52,6 +54,58 @@
 #'                boundaries.
 #'
 #'
+#' @details
+#' 1. Basic information for the function **DoSA()**:
+#' **DoSA()** supports sequential analysis of aggregate data synthesis based on
+#' head-to-head comparison using either binary or continuous data in each group.
+#' Minimum information for the function **DoSA()** encompasses a data set of
+#' study-level data, and information for analysis settings in terms of time
+#' sequence, presumed meaningful effect size, and presumed variance of the
+#' meaningful effect size. Operative points of using function **DoSA()** are listed
+#' below:
+#'
+#' 1.1. Parameter `data` should be used for assigning a data set.
+#' 1.2. Study-level data have to be assigned according to outcome type:
+#' 1.2.1. **For dichotomous outcome**: Parameter `n1` and `n2` should be defined
+#' with parameter `r1` and `r2`.
+#' 1.2.2. **For continuous outcome**: parameter `n1` and `n2` should be defined
+#' with parameter `m1`, `sd1`, `m2`, `sd2`.
+#' 1.3. Parameter `source`, `time`, `PES`, and `PV` are required for conducting
+#' sequential analysis. Other parameters are auxiliary.
+#'
+#' 2. Default in the function **DoSA()**
+#' Certain defaults have been elucidated in the introductory section about the
+#' parameters, but some of them need to be elaborated upon due to their complexity.
+#'
+#' 2.1. Default on the parameter `measure` is `"ES"` that automatically uses risk
+#' ratio ("RR") for binary outcome and mean difference ("MD") for continuous
+#' outcome respectively. Argument `"OR"` and `"SMD"` can be used for the parameter
+#' `measure` when original analysis pools data based on odds ratio or standardized
+#' mean difference.
+#'
+#' 2.2. Default on the parameter `method` is `"DL"` for applying DerSimonian-Laird
+#' heterogeneity estimator in the original pooled analysis. Other eligible arguments
+#' for the parameter are `"REML"` for restricted maximum-likelihood estimator,
+#' `"PM"` for Paule-Mandel estimator, `"ML"` for maximum-likelihood estimator,
+#' `"HS"` for Hunter-Schmidt estimator, `"SJ"` for Sidik-Jonkman estimator,
+#' `"HE"` for Hedges estimator, and `"EB"` for empirical Bayes estimator.
+#'
+#' 2.3. Default on the parameter `pooling` is `"IV"` for applying inverse variance
+#' weighting method. Other commonly-used and eligible arguments for the parameter
+#' are `"MH"` for Mantel-Haenszel method and `"Peto"` for pooling data using Peto
+#' method. The arguments `"MH"` and `"Peto"` are exclusively available for binary
+#' outcomes, while the argument `"IV"` will be automatically applied in the case
+#' of continuous outcomes.
+#'
+#' 2.4. Default on the parameter `adjust` is `"None"` for keeping the required
+#' information size (RIS) without adjustment. Other eligible arguments for the
+#' parameter are `"D2"` for adjusted RIS based on diversity (D-squared statistics),
+#' `"I2"` for adjusted RIS based on I-squared statistics, `"CHL"` for adjusted RIS
+#' based on low heterogeneity by multiplying 1.33, `"CHM"` for adjusted RIS by
+#' multiplying 2 due to moderate heterogeneity, and `"CHL"` for adjusted RIS by
+#' multiplying 4 due to high heterogeneity.
+#'
+#'
 #' @return
 #' **DoSA()** returns a summary on the result of sequential analysis.
 #' \item{studies}{Numbers of studies included in the sequential analysis.}
@@ -61,9 +115,10 @@
 #' \item{beta}{A numeric value of type II error for the sequential analysis.}
 #' \item{PES}{A numeric value of presumed meaningful effect size for the
 #'       sequential analysis.}
-#' \item{Varian}{A numeric value of presumed variance of the meaningful effect
+#' \item{RRR}{A numeric value of relative risk reduction.}
+#' \item{variance}{A numeric value of presumed variance of the meaningful effect
 #'       size for the sequential analysis.}
-#' \item{Divers}{A numeric value to show diversity in the pooled analysis.}
+#' \item{diversity}{A numeric value to show diversity in the pooled analysis.}
 #' \item{AF}{A numeric value of adjustment factor.}
 #' \item{RIS.org}{A numeric value for required information size without adjustment.}
 #' \item{RIS.adj}{A numeric value for adjusted required information size.}
@@ -95,7 +150,9 @@
 #' https://www.ncss.com/wp-content/themes/ncss/pdf/Procedures/NCSS/Group-Sequential_Analysis_for_Two_Proportions.pdf
 #'
 #'
-#' @seealso \code{\link{TestDisparity}}, \code{\link{TestDiscordance}}
+#' @seealso \code{\link{DoOSA}}, \code{\link{PlotOSA}},
+#'          \code{\link{TestDisparity}}, \code{\link{TestDiscordance}}
+#'
 #'
 #' @examples
 #' ## Not run:
@@ -112,40 +169,41 @@
 #'
 #' ## End(Not run)
 #'
+#'
 #' @export DoSA
 
 
 
-DoSA <- function(data = NULL,
-                 source = NULL,
-                 time = NULL,
-                 n = NULL,
-                 es = NULL,
-                 se = NULL,
-                 r1 = NULL,
-                 m1 = NULL,
-                 sd1 = NULL,
-                 n1 = NULL,
-                 r2 = NULL,
-                 m2 = NULL,
-                 sd2 = NULL,
-                 n2 = NULL,
-                 group = c("Group 1", "Group 2"),
-                 ref = 2,
-                 prefer = "small",
+DoSA <- function(data    = NULL,
+                 source  = NULL,
+                 time    = NULL,
+                 n       = NULL,
+                 es      = NULL,
+                 se      = NULL,
+                 r1      = NULL,
+                 m1      = NULL,
+                 sd1     = NULL,
+                 n1      = NULL,
+                 r2      = NULL,
+                 m2      = NULL,
+                 sd2     = NULL,
+                 n2      = NULL,
+                 group   = c("Group 1", "Group 2"),
+                 ref     = 2,
+                 prefer  = "small",
                  measure = "ES",
-                 model = "random",
-                 method = "DL",
+                 model   = "random",
+                 method  = "DL",
                  pooling = "IV",
-                 alpha = 0.05,
-                 beta = 0.2,
-                 PES = "post-hoc",
-                 PV = "post-hoc",
-                 adjust = "none",
-                 plot  = FALSE,
-                 id = FALSE,
-                 invert = FALSE,
-                 BSB = FALSE) {
+                 alpha   = 0.05,
+                 beta    = 0.2,
+                 PES     = "post-hoc",
+                 PV      = "post-hoc",
+                 adjust  = "none",
+                 plot    = FALSE,
+                 id      = FALSE,
+                 invert  = FALSE,
+                 BSB     = FALSE) {
 
   # 01. CHECK arguments -----
   lgcInData   <- ifelse(is.null(data), FALSE, TRUE)
@@ -173,16 +231,16 @@ DoSA <- function(data = NULL,
 
   lgcStop1     <- ifelse(lgcReq1 == TRUE, FALSE, TRUE)
   infoLgcStop1 <- ifelse(lgcStop1 == TRUE,
-                        'Argument "data" should be used for assigning a data set.',
+                        'Parameter "data" should be used for assigning a data set.',
                         "")
   lgcStop2     <- ifelse(TRUE %in% c(lgcReq2, lgcReq3, lgcReq4), FALSE, TRUE)
-  infoLgcStop2 <- ifelse(lgcStop2 == TRUE, 'Arguments "n", "es", and "se" should be defined for the analysis based on study-level data.
-                        Arguments "n1" and "n2" should be defined with "r1" and "r2" for dichotomous outcome based on arm-level data.
-                        Or arguments "n1" and "n2" should be defined with "m1", "sd1", "m2", "sd2" for continuous outcome based on arm-level data.',
+  infoLgcStop2 <- ifelse(lgcStop2 == TRUE, 'Parameter "n", "es", and "se" should be defined for the analysis based on study-level data.
+                        Parameter "n1" and "n2" should be defined with "r1" and "r2" for dichotomous outcome based on arm-level data.
+                        Or parameter "n1" and "n2" should be defined with "m1", "sd1", "m2", "sd2" for continuous outcome based on arm-level data.',
                         "")
   lgcStop3     <- ifelse(lgcReq5 == TRUE, FALSE, TRUE)
   infoLgcStop3 <- ifelse(lgcStop3 == FALSE,
-                        'Argument "source", "time", "PES", and "PV" are required',
+                        'Parameter "source", "time", "PES", and "PV" are required',
                         "")
 
   if (lgcStop1 | lgcStop2 | lgcStop3)
@@ -203,17 +261,17 @@ DoSA <- function(data = NULL,
   colnames(dataIn)[which(colnames(dataIn) == time)]   <- "time"
 
   if (lgcReq2) {
-    n      <- deparse(substitute(n))
-    es     <- deparse(substitute(es))
-    se     <- deparse(substitute(se))
+    n   <- deparse(substitute(n))
+    es  <- deparse(substitute(es))
+    se  <- deparse(substitute(se))
     colnames(dataIn)[which(colnames(dataIn) == n)]  <- "n"
     colnames(dataIn)[which(colnames(dataIn) == es)] <- "es"
     colnames(dataIn)[which(colnames(dataIn) == se)] <- "se"
   }
 
   if (lgcReq3) {
-    r1 <- deparse(substitute(r1))
-    r2 <- deparse(substitute(r2))
+    r1  <- deparse(substitute(r1))
+    r2  <- deparse(substitute(r2))
     colnames(dataIn)[which(colnames(dataIn) == r1)] <- "r1"
     colnames(dataIn)[which(colnames(dataIn) == r2)] <- "r2"
     if (measure == "ES") {
@@ -236,8 +294,8 @@ DoSA <- function(data = NULL,
   }
 
   if (TRUE %in% c(lgcReq3, lgcReq4)) {
-    n1 <- deparse(substitute(n1))
-    n2 <- deparse(substitute(n2))
+    n1  <- deparse(substitute(n1))
+    n2  <- deparse(substitute(n2))
     colnames(dataIn)[which(colnames(dataIn) == n1)] <- "n1"
     colnames(dataIn)[which(colnames(dataIn) == n2)] <- "n2"
     dataIn$n <- dataIn$n1 + dataIn$n2
@@ -251,9 +309,15 @@ DoSA <- function(data = NULL,
   infoMeasure  <- measure
   infoModel    <- model
   infoMethod   <- ifelse(infoModel == "random", method, "DL")
-  infoPooling  <- ifelse(pooling == "IV", "Inverse",
-                         ifelse(infoMeasure %in% c("RR", "OR"),
-                                pooling, "Inverse"))
+  infoPooling  <- ifelse(base::isFALSE(pooling %in% c("IV", "MH", "Peto")),
+                         ifelse(base::isFALSE(infoMeasure %in% c("MD", "SMD")),
+                                "MH", "Inverse"),
+                         ifelse(base::isFALSE(infoMeasure %in% c("MD", "SMD")),
+                                ifelse(pooling == "IV",
+                                       "Inverse",
+                                       pooling),
+                                "Inverse")
+                         )
   infoAlpha    <- alpha
   infoBeta     <- beta
   infoPES      <- PES
@@ -362,10 +426,12 @@ DoSA <- function(data = NULL,
       }
     }
 
+    infoRRR   <- abs(infoProp2 - infoProp1) / infoProp1
     infoESMA  <- infoProp2 - infoProp1
     infoVarMA <- (infoProp2 + infoProp1) / 2 * (1 - (infoProp2 + infoProp1) / 2)
     #infoSEMA  <- ifelse(infoModel == "random", outMA$seTE.random, outMA$seTE.fixed)
-  } else if (isFALSE(lgcReq3)) {
+  } else if (base::isFALSE(lgcReq3)) {
+    infoRRR   <- NA
     infoESMA  <- ifelse(infoModel == "random", outMA$TE.random, outMA$TE.fixed)
     infoSEMA  <- ifelse(infoModel == "random", outMA$seTE.random, outMA$seTE.fixed)
     #infoVarMA <- ifelse(infoModel == "random", 1 / sum(outMA$w.random), 1 / sum(outMA$w.fixed))
@@ -378,8 +444,23 @@ DoSA <- function(data = NULL,
   #infoSEMA  <- ifelse(infoModel == "random", outMA$seTE.random, outMA$seTE.fixed)
   #infoVarMA <- (infoSEMA * sqrt(infoCases - 2))^2
   infoPES   <- ifelse(infoPES == "post-hoc", infoESMA, infoPES)
-  infoPV    <- ifelse("post-hoc" %in% c(infoPES, infoPV), infoVarMA, infoPV)
-  infoI2    <- outMA$I2
+  #infoPV    <- ifelse("post-hoc" %in% c(infoPES, infoPV), infoVarMA, infoPV)
+  infoVarObs <- infoVarMA
+
+  if (infoPV == "post-hoc") {
+    infoPV <- infoVarMA
+  } else if (infoPV == "PES") {
+    # Variance of binary outcome based on presumed effect size -----------------
+    infoPV     <- infoVarMA
+    if (lgcReq3) {
+      infoRRR   <- abs(infoPES) / infoProp1
+      infoProp2 <- (infoPES / infoProp1 * infoProp1) + infoProp1
+      infoPV    <- (infoProp2 + infoProp1) / 2 * (1 - (infoProp2 + infoProp1) / 2)
+    }
+  }
+  #--------------------------------------------------------------------
+
+  infoI2 <- outMA$I2
 
   # 04. DO sequential analysis -----
   # alpha spending boundary (NCSS 710 Group-Sequential Analysis for Two Proportions 710-17)
@@ -513,7 +594,7 @@ DoSA <- function(data = NULL,
 
 
   ## 04.05 Adjusted MA -----
-  lgcMAAdj <- abs(dataSA$zCum[infoNumStud]) > abs(dataSA$asub[infoNumStud])
+  lgcMAAdj <- abs(dataSA$zCum[infoNumStud]) < abs(dataSA$asub[infoNumStud])
   infoLgcMAAdj  <- ifelse(lgcMAAdj,
                          "Adjusted confidence interval is suggested to be performed.",
                          "Adjusted confidence interval is not necessary to be performed.")
@@ -570,22 +651,23 @@ DoSA <- function(data = NULL,
 
 
   # 05. BUILD an DoSA object -----
-  lsDoSA <- list(name    = "Sequential analysis",
-                 studies = infoNumStud,
-                 AIS     = infoCases,
-                 alpha   = infoAlpha,
-                 beta    = infoBeta,
-                 PES     = infoPES,
-                 Pooing  = infoPooling,
-                 Varian  = infoPV,
-                 Divers  = infoDivers,
-                 AF      = ifelse(infoAdjust == "none",
-                                  "Undefined",
-                                  infoAF),
-                 RIS.org = ceiling(infoRISOrg),
-                 RIS.adj = ifelse(infoAdjust == "none",
-                                  "No adjusted",
-                                  ceiling(infoRISAdj))
+  lsDoSA <- list(name      = "Sequential analysis",
+                 studies   = infoNumStud,
+                 AIS       = infoCases,
+                 alpha     = infoAlpha,
+                 beta      = infoBeta,
+                 pooling   = infoPooling,
+                 PES       = infoPES,
+                 RRR       = ifelse(infoRRR < 0.001, "< 0.001", round(infoRRR, 3)),
+                 variance  = infoPV,
+                 diversity = infoDivers,
+                 AF        = ifelse(infoAdjust == "none",
+                                    "Undefined",
+                                    infoAF),
+                 RIS.org   = ceiling(infoRISOrg),
+                 RIS.adj   = ifelse(infoAdjust == "none",
+                                    "No adjusted",
+                                    ceiling(infoRISAdj))
                  )
   class(lsDoSA)    <- c("aides", "DoSA")
   lsDoSA$frag      <- paste(round(dataSA$frag[infoNumStud], 4) * 100,
@@ -609,7 +691,7 @@ DoSA <- function(data = NULL,
 
 
   # 06. RETURN summary of function `DoSA()` -----
-  cat(paste("\n"), fill = TRUE, sep = "")
+  #cat(paste("\n"), fill = TRUE, sep = "")
   cat(paste("Summary of sequential analysis (main information)\n",
             " Acquired sample size: ",
             infoCases,
@@ -664,7 +746,10 @@ DoSA <- function(data = NULL,
                          round(infoProp1, 10) * 100,
                          "%, and ",
                          round(infoProp2, 10) * 100,
-                         "% respectively)",
+                         "% respectively; RRR ",
+                         ifelse(infoRRR < 0.001, "< 0.001)",
+                                paste("= ", round(infoRRR, 3), ")",
+                                      sep = "")),
                          sep = ""),
                    ""
                    ),
